from PIL import Image, ImageDraw, ImageFont
from typing import Optional, Tuple, Union, Dict, List
from pathlib import Path
import random
try:
    import ujson as json
except ModuleNotFoundError:
    import json

from .config import fortune_config

def get_copywriting() -> Tuple[str, str]:
    '''
        Read the copywriting.json, choice a luck with a random content
    '''
    _p: Path = fortune_config.fortune_path / "fortune" / "copywriting.json"
    content: List[Dict[str, Union[str, int, List[str]]]] = {}

    with open(_p, "r", encoding="utf-8") as f:
        content = json.load(f).get("copywriting")
        
    luck: Dict[str, Union[str, int, List[str]]] = random.choice(content)
    
    title: str = luck.get("good-luck")
    text: str = random.choice(luck.get("content"))

    return title, text

def randomBasemap(_theme: str, _spec_path: Optional[str]) -> Path:
    if isinstance(_spec_path, str):
        p: Path = fortune_config.fortune_path / "img" / _spec_path
        return p

    if _theme == "random":
        __p: Path = fortune_config.fortune_path / "img"
        # Each dir is a theme, remember add _flag after the names of themes
        themes: List[str] = [f.name for f in __p.iterdir() if f.is_dir() and theme_flag_check(f.name + "_flag")]
        picked = random.choice(themes)

        _p: Path = __p / picked
        # Each file is a posix path of images
        images: List[Path] = [i for i in _p.iterdir() if i.is_file()]
        p: Path = random.choice(images)
    else:
        _p: Path = fortune_config.fortune_path / "img" / _theme
        images: List[Path] = [i for i in _p.iterdir() if i.is_file()]
        p: Path = random.choice(images)
    
    return p

def drawing(_theme: str, _spec_path: Optional[str], gid: str, uid: str) -> Path:
    # 1. Random choice a base image
    imgPath: Path = randomBasemap(_theme, _spec_path)
    img: Image.Image = Image.open(imgPath)
    draw = ImageDraw.Draw(img)
    
    # 2. Random choice a luck text with title
    title, text = get_copywriting()
    
    # 3. Draw
    font_size = 45
    color = "#F5F5F5"
    image_font_center = (140, 99)
    fontPath = {
        "title": f"{fortune_config.fortune_path}/font/Mamelon.otf",
        "text": f"{fortune_config.fortune_path}/font/sakura.ttf",
    }
    ttfront = ImageFont.truetype(fontPath["title"], font_size)
    font_length = ttfront.getsize(title)
    draw.text(
        (
            image_font_center[0] - font_length[0] / 2,
            image_font_center[1] - font_length[1] / 2,
        ),
        title,
        fill=color,
        font=ttfront,
    )
    # Text rendering
    font_size = 25
    color = "#323232"
    image_font_center = [140, 297]
    ttfront = ImageFont.truetype(fontPath["text"], font_size)
    result = decrement(text)
    
    textVertical = []
    for i in range(0, result[0]):
        font_height = len(result[i + 1]) * (font_size + 4)
        textVertical = vertical(result[i + 1])
        x = int(
            image_font_center[0]
            + (result[0] - 2) * font_size / 2
            + (result[0] - 1) * 4
            - i * (font_size + 4)
        )
        y = int(image_font_center[1] - font_height / 2)
        draw.text((x, y), textVertical, fill=color, font=ttfront)
    # Save
    outPath = exportFilePath(imgPath, gid, uid)
    img.save(outPath)
    return outPath

def exportFilePath(originalFilePath: Path, gid: str, uid: str) -> Path:
    dirPath: Path = fortune_config.fortune_path / "out"
    if not dirPath.exists():
        dirPath.mkdir(exist_ok=True, parents=True)

    outPath: Path = originalFilePath.parent.parent.parent / "out" / f"{gid}_{uid}.png" 
    return outPath

def decrement(text: str) -> List[str]:
    length = len(text)
    result = []
    cardinality = 9
    if length > 4 * cardinality:
        raise Exception
    
    numberOfSlices = 1
    while length > cardinality:
        numberOfSlices += 1
        length -= cardinality
        
    result.append(numberOfSlices)
    
    # Optimize for two columns
    space = " "
    length = len(text)
    if numberOfSlices == 2:
        if length % 2 == 0:
            # even
            fillIn = space * int(9 - length / 2)
            return [
                numberOfSlices,
                text[: int(length / 2)] + fillIn,
                fillIn + text[int(length / 2) :],
            ]
        else:
            # odd number
            fillIn = space * int(9 - (length + 1) / 2)
            return [
                numberOfSlices,
                text[: int((length + 1) / 2)] + fillIn,
                fillIn + space + text[int((length + 1) / 2) :],
            ]
            
    for i in range(0, numberOfSlices):
        if i == numberOfSlices - 1 or numberOfSlices == 1:
            result.append(text[i * cardinality :])
        else:
            result.append(text[i * cardinality : (i + 1) * cardinality])
            
    return result

def vertical(_str: List[str]) -> str:
    _list = []
    for s in _str:
        _list.append(s)
    return "\n".join(_list)

def theme_flag_check(_theme: str) -> bool:
    '''
        Read the config json, return the status of a theme
    '''
    flag_config_path: Path = fortune_config.fortune_path / "fortune_config.json"
    with flag_config_path.open("r", encoding="utf-8") as f:
        data: Dict[str, bool] = json.load(f)
    
    return data.get((_theme + "_flag"), False)

__all__ = [
    drawing, theme_flag_check
]


from nonebot import logger, on_command, on_regex, on_fullmatch
from nonebot.params import Depends, State, CommandArg, RegexMatched
from nonebot.typing import T_State
from nonebot.permission import SUPERUSER
from nonebot.matcher import Matcher
from nonebot.adapters.onebot.v11 import GROUP, GROUP_ADMIN, GROUP_OWNER, Message, GroupMessageEvent, MessageSegment
from .data_source import fortune_manager
from .config import MainThemeList
from nonebot_plugin_apscheduler import scheduler

__fortune_version__ = "v0.4.7"
__fortune_notes__ = f'''
今日运势 {__fortune_version__}
[今日运势/抽签/运势] 抽签
[xx抽签]    指定主题抽签
[指定xx签] 指定特殊角色签底，需要自己尝试哦~
[设置xx签] 设置群抽签主题
[重置主题] 重置群抽签主题
[主题列表] 查看可选的抽签主题
[查看主题] 查看群抽签主题'''.strip()

divine = on_command("今日运势", aliases={"抽签", "运势"}, permission=GROUP, priority=8)
divine_specific = on_regex(r"^.+抽签$", permission=GROUP, priority=8)
limit_setting = on_regex(r"^指定(.*?)签$", permission=GROUP, priority=8)
theme_setting = on_regex(r"^设置(.*?)签$", permission=SUPERUSER | GROUP_ADMIN | GROUP_OWNER, priority=8, block=True)
reset = on_regex("^重置(抽签)?主题$", permission=SUPERUSER | GROUP_ADMIN | GROUP_OWNER, priority=8, block=True)
theme_list = on_fullmatch("主题列表", permission=GROUP, priority=8, block=True)
show = on_regex("^查看(抽签)?主题$", permission=GROUP, priority=8, block=True)
refresh = on_fullmatch("刷新抽签", permission=SUPERUSER, priority=8, block=True)

@show.handle()
async def _(event: GroupMessageEvent):
    gid = str(event.group_id)
    theme = fortune_manager.get_setting(gid)
    show_theme = MainThemeList[theme][0]
    await show.finish(f"当前群抽签主题：{show_theme}")

@theme_list.handle()
async def _(matcher: Matcher):
    msg = fortune_manager.get_main_theme_list()
    await matcher.finish(msg)

@divine.handle()
async def _(event: GroupMessageEvent, args: Message = CommandArg()):
    args = args.extract_plain_text()
    
    if "帮助" in args[-2:]:
        await divine.finish(__fortune_notes__)
    
    gid = str(event.group_id)
    uid = str(event.user_id)
    nickname = event.sender.card if event.sender.card else event.sender.nickname
    
    image_file, status = fortune_manager.divine(None, None, gid, uid, nickname)
    if image_file is False:
        await divine.finish("今日运势生成出错……") 
    
    if not status:
        msg = MessageSegment.text("你今天抽过签了，再给你看一次哦🤗\n") + MessageSegment.image(image_file)
    else:
        logger.info(f"User {event.user_id} | Group {event.group_id} 占卜了今日运势")
        msg = MessageSegment.text("✨今日运势✨\n") + MessageSegment.image(image_file)
    
    await divine.finish(msg, at_sender=True)        

async def get_user_theme(matcher: Matcher, args: str = RegexMatched(), state: T_State = State()):
    arg = args[:-2]
    if len(arg) < 1:
        await matcher.finish("输入参数错误")
        
    return {**state, "user_theme": arg}

@divine_specific.handle()
async def _(event: GroupMessageEvent, state: T_State = Depends(get_user_theme)):
    user_theme = state["user_theme"]
    
    for theme in MainThemeList:
        if user_theme in MainThemeList[theme]:
            if not fortune_manager.theme_enable_check(theme):
                await divine_specific.finish("该抽签主题未启用~")
            else:
                gid = str(event.group_id)
                uid = str(event.user_id)
                nickname = event.sender.card if event.sender.card else event.sender.nickname
                
                image_file, status = fortune_manager.divine(theme, None, gid, uid, nickname)
                if image_file is False:
                    await divine_specific.finish("今日运势生成出错……") 
        
                if not status:
                    msg = MessageSegment.text("你今天抽过签了，再给你看一次哦🤗\n") + MessageSegment.image(image_file)
                else:
                    logger.info(f"User {event.user_id} | Group {event.group_id} 占卜了今日运势")
                    msg = MessageSegment.text("✨今日运势✨\n") + MessageSegment.image(image_file)
            
            await divine_specific.finish(msg, at_sender=True)

    await divine_specific.finish("还没有这种抽签主题哦~")

async def get_user_arg(matcher: Matcher, args: str = RegexMatched(), state: T_State = State()):
    arg = args[2:-1]
    if len(arg) < 1:
        await matcher.finish("输入参数错误")
        
    return {**state, "user_arg": arg}
        
@theme_setting.handle()
async def _(event: GroupMessageEvent, state: T_State = Depends(get_user_arg)):
    user_theme = state["user_arg"]
    gid = str(event.group_id)
    
    for theme in MainThemeList:
        if user_theme in MainThemeList[theme]:
            if not fortune_manager.divination_setting(theme, gid):
                await theme_setting.finish("该抽签主题未启用~")
            else:
                await theme_setting.finish("已设置当前群抽签主题~")

    await theme_setting.finish("还没有这种抽签主题哦~")

@limit_setting.handle()
async def _(event: GroupMessageEvent, state: T_State = Depends(get_user_arg)):
    '''
        指定签底抽签功能将在v0.5.x版本中弃用，但会保留在v0.4.x，届时请查看README说明
    '''
    logger.warning("The command of divining by indicating the basic image of a specific theme will be deprecated in version v0.5.x in the future, but will be reserved in v0.4.x")
    
    limit = state["user_arg"]
    gid = str(event.group_id)
    uid = str(event.user_id)
    nickname = event.sender.card if event.sender.card else event.sender.nickname
    
    if limit == "随机":
        image_file, status = fortune_manager.divine(None, None, gid, uid, nickname)
        if image_file is False:
            await limit_setting.finish("今日运势生成出错……") 
    else:
        spec_path = fortune_manager.limit_setting_check(limit)
        if not spec_path:
            await limit_setting.finish("还不可以指定这种签哦，请确认该签底对应主题开启或图片路径存在~")
        else:
            image_file, status = fortune_manager.divine(None, spec_path, gid, uid, nickname)
            if image_file is False:
                await limit_setting.finish("今日运势生成出错……") 
        
    if not status:
        msg = MessageSegment.text("你今天抽过签了，再给你看一次哦🤗\n") + MessageSegment.image(image_file)
    else:
        logger.info(f"User {event.user_id} | Group {event.group_id} 占卜了今日运势")
        msg = MessageSegment.text("✨今日运势✨\n") + MessageSegment.image(image_file)
    
    await limit_setting.finish(msg, at_sender=True)

@reset.handle()
async def _(event: GroupMessageEvent):
    gid = str(event.group_id)
    fortune_manager.divination_setting("random", gid)
    await reset.finish("已重置当前群抽签主题为随机~")
    
@refresh.handle()
async def _():
    fortune_manager.reset_fortune()
    await limit_setting.finish("今日运势已刷新!")

# 重置每日占卜
@scheduler.scheduled_job("cron", hour=0, minute=0, misfire_grace_time=60)
async def _():
    fortune_manager.reset_fortune()
    logger.info("今日运势已刷新！")


from nonebot import logger
from typing import Union
import httpx

class DownloadError(Exception):
    def __init__(self, msg):
        self.msg = msg
        
    def __str__(self):
        return self.msg

async def download_url(url: str) -> Union[httpx.Response, None]:
    async with httpx.AsyncClient() as client:
        for i in range(3):
            try:
                resp = await client.get(url, timeout=10)
                if resp.status_code != 200:
                    continue
                return resp
            except Exception:
                logger.warning(f"Error occurred when downloading {url}, retry: {i}/3")
    
    logger.warning(f"Abort downloading")
    return None


from nonebot import get_driver, logger
from pydantic import BaseModel, Extra
from typing import List, Dict
from pathlib import Path
try:
    import ujson as json
except ModuleNotFoundError:
    import json
    
from .download import DownloadError, download_url

'''
    抽签主题对应表，第一键值为“抽签设置”或“主题列表”展示的主题名称
    Key-Value: 主题资源文件夹名-设置主题别名
'''
MainThemeList: Dict[str, List[str]] = {
    "random": ["随机"],
    "pcr": ["PCR", "公主链接", "公主连结", "Pcr", "pcr"],
    "genshin": ["原神", "Genshin Impact", "genshin", "Genshin", "op", "原批"],
    "hololive": ["Hololive", "hololive", "Vtb", "vtb", "管人", "holo", "猴楼"],
    "touhou": ["东方", "touhou", "Touhou", "车万"],
    "touhou_lostword": ["东方归言录", "东方lostword", "touhou lostword", "Touhou dlc"],
    "touhou_old": ["旧东方", "旧版东方", "老东方", "老版东方", "经典东方"],
    "onmyoji": ["阴阳师", "yys", "Yys", "痒痒鼠"],
    "azure": ["碧蓝航线", "碧蓝", "azure", "Azure"],
    "asoul": ["Asoul", "asoul", "a手", "A手", "as", "As"],
    "arknights": ["明日方舟", "方舟", "arknights", "鹰角", "Arknights", "舟游"],
    "granblue_fantasy": ["碧蓝幻想", "Granblue Fantasy", "granblue fantasy", "幻想", "fantasy", "Fantasy"],
    "punishing":["战双", "战双帕弥什"],
    "pretty_derby": ["赛马娘", "马", "马娘", "赛马"],
    "dc4": ["dc4", "DC4", "Dc4", "初音岛", "初音岛4"],
    "einstein": ["爱因斯坦携爱敬上", "爱因斯坦", "einstein", "Einstein"],
    "sweet_illusion": ["灵感满溢的甜蜜创想", "甜蜜一家人", "富婆妹"],
    "liqingge": ["李清歌", "清歌"],
    "hoshizora": ["星空列车与白的旅行", "星空列车"],
    "sakura": ["樱色之云绯色之恋", "樱云之恋", "樱云绯恋", "樱云"],
    "summer_pockets": ["夏日口袋", "夏兜", "sp", "SP"],
    "amazing_grace": ["奇异恩典"]
}

class ResourceError(Exception):
    def __init__(self, msg):
        self.msg = msg
        
    def __str__(self):
        return self.msg

class PluginConfig(BaseModel, extra=Extra.ignore):
    fortune_path: Path = Path(__file__).parent / "resource"

class ThemeFlagConfig(BaseModel, extra=Extra.ignore):
    '''
        各主题抽签开关，仅在random抽签中生效
        请确保不全是False
    '''
    amazing_grace_flag: bool = True
    arknights_flag: bool = True
    asoul_flag: bool = True
    azure_flag: bool = True
    genshin_flag:  bool = True
    onmyoji_flag: bool = True
    pcr_flag: bool = True
    touhou_flag: bool = True
    touhou_lostword_flag: bool = True
    touhou_old_flag: bool = True
    hololive_flag: bool = True
    granblue_fantasy_flag: bool = True
    punishing_flag: bool = True
    pretty_derby_flag: bool = True
    dc4_flag: bool = True
    einstein_flag: bool = True
    sweet_illusion_flag: bool = True
    liqingge_flag: bool = True
    hoshizora_flag: bool = True
    sakura_flag: bool = True 
    summer_pockets_flag: bool = True

driver = get_driver()
fortune_config: PluginConfig = PluginConfig.parse_obj(driver.config.dict())
theme_flag_config: ThemeFlagConfig = ThemeFlagConfig.parse_obj(driver.config.dict())

@driver.on_startup
async def fortune_check() -> None:
    flag_config_path: Path = fortune_config.fortune_path / "fortune_config.json"
    
    if not fortune_config.fortune_path.exists():
        fortune_config.fortune_path.mkdir(parents=True, exist_ok=True)
    
    '''
        Check whether all themes disable
    '''
    content = theme_flag_config.dict()
    _flag: bool = False
    for theme in content:
        if content.get(theme, False):
            _flag = True
            break
    
    if not _flag:
        logger.warning("Fortune themes ALL disabled! Please check!")
        raise ResourceError("Fortune themes ALL disabled! Please check!")
                
    with flag_config_path.open("w", encoding="utf-8") as f:
        json.dump(content, f, ensure_ascii=False, indent=4)
    
    '''
        Try to get the latest copywriting from repo
    '''
    copywriting_path: Path = fortune_config.fortune_path / "fortune" / "copywriting.json"
    url = "https://raw.fastgit.org/MinatoAquaCrews/nonebot_plugin_fortune/beta/nonebot_plugin_fortune/resource/fortune/copywriting.json"
    response = await download_url(url)
    if response is None:
        if not copywriting_path.exists():
           raise DownloadError("Copywriting resource missing! Please check!")
    else:
        docs = response.json()
        version = docs.get("version")

        with copywriting_path.open("w", encoding="utf-8") as f:
            json.dump(docs, f, ensure_ascii=False, indent=4)
        
        logger.info(f"Get the latest copywriting docs from repo, version: {version}")
    
    fortune_data_path: Path = fortune_config.fortune_path / "fortune_data.json"
    fortune_setting_path: Path = fortune_config.fortune_path / "fortune_setting.json"
    
    if not fortune_data_path.exists():
        logger.warning("fortune_data.json is missing, but initialized one...")
        
        with fortune_data_path.open("w", encoding="utf-8") as f:
            json.dump(dict(), f, ensure_ascii=False, indent=4)

    if not fortune_setting_path.exists():
        url = "https://raw.fastgit.org/MinatoAquaCrews/nonebot_plugin_fortune/beta/nonebot_plugin_fortune/resource/fortune_setting.json"
        response = await download_url(url)
        if response is None:
            logger.warning("fortune_setting.json is missing, but initialized one...")
            
            content = {
                "group_rule": {}, 
                "specific_rule": {}
            }
            with fortune_setting_path.open("w", encoding="utf-8") as f:
                json.dump(content, f, ensure_ascii=False, indent=4)
        else:
            setting = response.json()

            with fortune_setting_path.open("w", encoding="utf-8") as f:
                json.dump(setting, f, ensure_ascii=False, indent=4)
            
            logger.info(f"fortune_setting.json is missing, but downloaded from repo")


from typing import Optional, Union, Tuple, List, Dict
from pathlib import Path
import random
try:
    import ujson as json
except ModuleNotFoundError:
    import json

from .config import fortune_config, MainThemeList
from .utils import drawing, theme_flag_check

class FortuneManager:
    def __init__(self):
        self._user_data: Dict[str, Dict[str, Dict[str, Union[str, bool]]]] = {}
        self._setting: Dict[str, Union[str, Dict[str, List[str]]]] = {}
        self._data_file: Path = fortune_config.fortune_path / "fortune_data.json"
        self._setting_file: Path = fortune_config.fortune_path / "fortune_setting.json"
    
    def _multi_divine_check(self, gid: str, uid: str) -> bool:
        '''
            检测是否重复抽签
        '''
        self._load_data()
        
        return self._user_data[gid][uid]["is_divined"]
    
    def limit_setting_check(self, limit: str) -> Union[bool, str]:
        '''
            检测是否有该特定规则
            检查指定规则的签底所对应主题是否开启或路径是否存在
        '''
        self._load_setting()
        
        if not self._setting["specific_rule"].get(limit):
            return False
        
        spec_path = random.choice(self._setting["specific_rule"][limit])
        for theme in MainThemeList:
            if theme in spec_path:
                return spec_path if theme_flag_check(theme) else False
        
        return False

    def divine(self, _theme: Optional[str], _spec_path: Optional[str], gid: str, uid: str, nickname: str) -> Tuple[Union[Path, bool], bool]:
        '''
            今日运势抽签
            主题在群设置主题divination__setting()已确认合法
        '''
        self._init_user_data(gid ,uid, nickname)
        
        self._load_setting()
        if not isinstance(_theme, str):
            theme = self._setting["group_rule"][gid]
        else:
            theme = _theme
            
        if not self._multi_divine_check(gid, uid):
            try:
                image_file = drawing(theme, _spec_path, gid, uid)
            except Exception:
                return False, True
            
            self._end_data_handle(gid, uid)
            return image_file, True
        else:
            image_file = fortune_config.fortune_path / "out" / f"{gid}_{uid}.png"
            return image_file, False

    def reset_fortune(self) -> None:
        '''
            重置今日运势并清空图片
        '''
        self._load_data()
        for gid in self._user_data:
            for uid in list(self._user_data[gid]):
                if self._user_data[gid][uid]["is_divined"] == False:
                    self._user_data[gid].pop(uid)
                else:
                    self._user_data[gid][uid]["is_divined"] = False
        
        self._save_data()

        dirPath: Path = fortune_config.fortune_path / "out"
        for pic in dirPath.iterdir():
            pic.unlink()

    def _init_user_data(self, gid: str, uid: str, nickname: str) -> None:
        '''
            初始化用户信息
        '''
        self._load_data()
        self._load_setting()
        
        if "group_rule" not in self._setting:
            self._setting["group_rule"] = {}
        if "specific_rule" not in self._setting:
            self._setting["specific_rule"] = {}
        if gid not in self._setting["group_rule"]:
            self._setting["group_rule"][gid] = "random"
        if gid not in self._user_data:
            self._user_data[gid] = {}
        if uid not in self._user_data[gid]:
            self._user_data[gid][uid] = {
                "gid": gid,
                "uid": uid,
                "nickname": nickname,
                "is_divined": False
            }
        
        self._save_data()
        self._save_setting()

    def get_main_theme_list(self) -> str:
        '''
            获取可设置的抽签主题
        '''
        msg = "可选抽签主题"
        for theme in MainThemeList:
            if theme != "random" and theme_flag_check(theme):
                msg += f"\n{MainThemeList[theme][0]}"
        
        return msg

    def _end_data_handle(self, gid: str, uid: str) -> None:
        '''
            占卜结束数据保存
        '''
        self._load_data()
        
        self._user_data[gid][uid]["is_divined"] = True
        self._save_data()
    
    def theme_enable_check(self, _theme: str) -> bool:
        '''
            Check whether a theme is enable
        '''
        return _theme == "random" or theme_flag_check(_theme)

    def divination_setting(self, theme: str, gid: str) -> bool:
        '''
            分群管理抽签设置
        '''
        self._load_setting()
        
        if self.theme_enable_check(theme):
            self._setting["group_rule"][gid] = theme
            self._save_setting()
            return True
        
        return False

    def get_setting(self, gid: str) -> str:
        '''
            获取当前群抽签主题，若没有数据则置随机
        '''
        self._load_setting()
        
        if gid not in self._setting["group_rule"]:
            self._setting["group_rule"][gid] = "random"
            self._save_setting()

        return self._setting["group_rule"][gid]
    
    # ------------------------------ Utils ------------------------------ #
    def _load_setting(self) -> None:
        '''
            读取各群抽签设置
        '''
        with open(self._setting_file, 'r', encoding='utf-8') as f:
            self._setting = json.load(f)

    def _save_setting(self) -> None:
        '''
            保存各群抽签设置
        '''
        with open(self._setting_file, 'w', encoding='utf-8') as f:
            json.dump(self._setting, f, ensure_ascii=False, indent=4)
            
    def _load_data(self) -> None:
        '''
            读取抽签数据
        '''
        with open(self._data_file, 'r', encoding='utf-8') as f:
            self._user_data = json.load(f)

    def _save_data(self) -> None:
        '''
            保存抽签数据
        '''
        with open(self._data_file, 'w', encoding='utf-8') as f:
            json.dump(self._user_data, f, ensure_ascii=False, indent=4)

fortune_manager = FortuneManager()

__all__ = [
    fortune_manager
]


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nonebot_plugin_fortune']

package_data = \
{'': ['*'],
 'nonebot_plugin_fortune': ['resource/*',
                            'resource/font/*',
                            'resource/fortune/*']}

install_requires = \
['httpx>=0.23.0,<0.24.0',
 'nonebot-adapter-onebot>=2.1.1,<3.0.0',
 'nonebot-plugin-apscheduler>=0.1.3,<0.2.0',
 'nonebot2>=2.0.0b3,<3.0.0',
 'pillow>=9.0.0,<10.0.0',
 'ujson>=5.1.0,<6.0.0']

setup_kwargs = {
    'name': 'nonebot-plugin-fortune',
    'version': '0.4.7',
    'description': 'Fortune divination!',
    'long_description': '<div align="center">\n\n# Fortune\n\n<!-- prettier-ignore-start -->\n<!-- markdownlint-disable-next-line MD036 -->\n_🙏 今日运势 🙏_\n<!-- prettier-ignore-end -->\n\n</div>\n<p align="center">\n  \n  <a href="https://github.com/MinatoAquaCrews/nonebot_plugin_fortune/blob/beta/LICENSE">\n    <img src="https://img.shields.io/github/license/MinatoAquaCrews/nonebot_plugin_fortune?color=blue">\n  </a>\n  \n  <a href="https://github.com/nonebot/nonebot2">\n    <img src="https://img.shields.io/badge/nonebot2-2.0.0b3+-green">\n  </a>\n  \n  <a href="https://github.com/MinatoAquaCrews/nonebot_plugin_fortune/releases/tag/v0.4.7">\n    <img src="https://img.shields.io/github/v/release/MinatoAquaCrews/nonebot_plugin_fortune?color=orange">\n  </a>\n\n  <a href="https://www.codefactor.io/repository/github/MinatoAquaCrews/nonebot_plugin_fortune">\n    <img src="https://img.shields.io/codefactor/grade/github/MinatoAquaCrews/nonebot_plugin_fortune/beta?color=red">\n  </a>\n  \n</p>\n\n## 版本\n\nv0.4.7 运势文案！全新的！\n\n👉 [如何在v0.4.2或更早版本上更新抽签主题资源？](https://github.com/MinatoAquaCrews/nonebot_plugin_fortune/blob/beta/How-to-add-new-theme.md)\n\n⚠ 适配nonebot2-2.0.0b3+\n\n[更新日志](https://github.com/MinatoAquaCrews/nonebot_plugin_fortune/releases/tag/v0.4.7)\n\n## 安装\n\n1. 安装方式：\n\n    - 通过`pip`或`nb`；pypi无法发行过大安装包，由此安装的插件不包含`resource/img`下所有**抽签主题图片**，需单独下载，建议`zip`包下载后单独提取`resource/img`抽签主题图片，后更改`FORTUNE_PATH`配置即可；\n    \n    - 通过`zip`或`git clone`安装：包含`resource`下所有主题抽签资源；\n\n2. 抽签主题图片`img`、字体`font`、文案`fortune`等资源均位于`./resource`下，可在`env`中设置`FORTUNE_PATH`；\n\n    ```python\n    FORTUNE_PATH="your_path_to_resource"  # For example, "./my-data/fortune"，其下有img、font、fortune文件夹等资源\n    ```\n\n3. 使用[FloatTech-zbpdata/Fortune](https://github.com/FloatTech/zbpdata)全部主题。在`env`下设置`xxx_FLAG`以启用或关闭抽签随机主题（默认全部开启），例如：\n\n    ```python\n    ARKNIGHTS_FLAG=true         # 明日方舟\n    ASOUL_FLAG=false            # A-SOUL\n    AZURE_FLAG=true             # 碧蓝航线\n    GENSHIN_FLAG=true           # 原神\n    ONMYOJI_FLAG=false          # 阴阳师\n    PCR_FLAG=true               # 公主连结\n    TOUHOU_FLAG=true            # 东方\n    TOUHOU_LOSTWORD_FLAG=true   # 东方归言录\n    TOUHOU_OLD_FLAG=false       # 东方旧版\n    HOLOLIVE_FLAG=true          # Hololive\n    PUNISHING_FLAG=true         # 战双帕弥什\n    GRANBLUE_FANTASY_FLAG=true  # 碧蓝幻想\n    PRETTY_DERBY_FLAG=true      # 赛马娘\n    DC4_FLAG=false              # dc4\n    EINSTEIN_FLAG=true          # 爱因斯坦携爱敬上\n    SWEET_ILLUSION_FLAG=true    # 灵感满溢的甜蜜创想\n    LIQINGGE_FLAG=true          # 李清歌\n    HOSHIZORA_FLAG=true         # 星空列车与白的旅行\n    SAKURA_FLAG=true            # 樱色之云绯色之恋\n    SUMMER_POCKETS_FLAG=true    # 夏日口袋\n    AMAZING_GRACE_FLAG=true     # 奇异恩典·圣夜的小镇\n    ```\n\n    **请确保不全为`false`，否则会抛出错误**\n\n4. 在`./resource/fortune_setting.json`内配置**指定抽签**规则，例如：\n\n    ```json\n    {\n        "group_rule": {\n            "123456789": "random",\n            "987654321": "azure",\n            "123454321": "granblue_fantasy"\n        },\n        "specific_rule": {\n            "凯露": [\n                "pcr\\/frame_1.jpg",\n                "pcr\\/frame_2.jpg"\n            ],\n            "可可萝": [\n                "pcr\\/frame_41.jpg"\n            ]\n        }\n    }\n    ```\n\n    *group_rule会自动生成，specific_rule可手动配置*\n\n    ⚠ 将在`v0.5.x`弃用\n\n    指定凯露签，由于存在两张凯露的签底，配置凯露签的**路径列表**即可，其余类似，**请确保图片路径、格式输入正确**；\n\n5. 占卜一下你的今日运势！🎉\n\n## 功能\n\n1. 随机抽取今日运势，配置多种抽签主题：原神、PCR、Hololive、东方、东方归言录、明日方舟、旧版东方、赛马娘、阴阳师、碧蓝航线、碧蓝幻想、战双帕弥什，galgame主题等……\n\n2. 可指定主题抽签；\n\n3. 每群每人一天限抽签1次，0点刷新（贪心的人是不会有好运的🤗）抽签信息并清除`./resource/out`下图片；\n\n4. 抽签的信息会保存在`./resource/fortune_data.json`内；群抽签设置及指定抽签规则保存在`./resource/fortune_setting.json`内；抽签生成的图片当天会保存在`./resource/out`下；\n\n5. `fortune_setting.json`已预置明日方舟、Asoul、原神、东方、Hololive、李清歌的指定抽签规则；\n\n6. 🔥 **重磅更新** 全新的运势文案！原`goodLuck.json`已移除，现`copywriting.json`整合了19种运势及共计600+条文案！\n\n\t⚠ `version`字段记录文案版本，后续版本将实现从repo自动更新最新文案资源\n\n\t⚠ 文案资源来自于hololive早安系列2019年第6.10～9.9期，有修改。\n\n7. **新增** 抽签主题启用检查，当全部为`false`会抛出错误。\n\n8. TODO in `v0.5.x` 🥳\n\n\t- [ ] 优化设置主题、指定主题、及检索的方式；\n\t- [ ] 文案排版算法；\n\t- [ ] 新增功能：每日星座运势；\n\t- [ ] 新增功能：资源缺失检查、自动下载：目前会尝试自动从repo中下载最新的`copywriting.json`；\n\t- [ ] 新增资源：新的抽签主题资源！\n\n## 命令\n\n1. 一般抽签：今日运势、抽签、运势；\n\n2. 指定主题抽签：[xx抽签]，例如：PCR抽签、holo抽签；[#18](https://github.com/MinatoAquaCrews/nonebot_plugin_fortune/issues/18)\n\n3. 指定签底并抽签：指定[xxx]签，在`./resource/fortune_setting.json`内手动配置；\n\n\t⚠ 将在`v0.5.x`弃用\n\n4. [群管或群主或超管] 配置抽签主题：\n\n    - 设置[原神/pcr/东方/vtb/xxx]签：设置群抽签主题；\n\n    - **修改** 重置（抽签）主题：设置群抽签主题为随机；\n\n5. 抽签设置：查看当前群抽签主题的配置；\n\n6. [超管] 刷新抽签：全局即刻刷新抽签，防止过0点未刷新的意外（虽然这个问题已经得到修复）；\n\n7. 今日运势帮助：显示插件帮助文案；\n\n8. **修改** 查看（抽签）主题：显示当前已启用主题；\n\n## 效果\n\n测试效果出自群聊。\n\n![display](./display.jpg)\n\n## 本插件改自\n\n[opqqq-plugin](https://github.com/opq-osc/opqqq-plugin)\n\n## 抽签图片及文案资源\n\n1. [opqqq-plugin](https://github.com/opq-osc/opqqq-plugin)：原神、pcr、hololive抽签主题；\n\n2. 感谢江樂丝提供东方签底；\n\n3. 东方归言录(touhou_lostword)：[KafCoppelia](https://github.com/KafCoppelia)；\n\n4. [FloatTech-zbpdata/Fortune](https://github.com/FloatTech/zbpdata)：其余主题签；\n\n5. 新版运势文案资源：[KafCoppelia](https://github.com/KafCoppelia)。`copywriting.json`整合了関係運、全体運、勉強運、金運、仕事運、恋愛運、総合運、大吉、中吉、小吉、吉、半吉、末吉、末小吉、凶、小凶、半凶、末凶、大凶及600+条运势文案！来自于hololive早安系列2019年第6.10～9.9期，有修改。',
    'author': 'KafCoppelia',
    'author_email': 'k740677208@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.3,<4.0.0',
}


setup(**setup_kwargs)
